
use std::cell::RefCell;

use gtk::glib::{self, closure_local};
use gtk::glib::subclass::InitializingObject;
use gtk::prelude::{BoxExt, ButtonExt, Cast, CastNone, ObjectExt, OrientableExt, PopoverExt, WidgetExt};
use gtk::subclass::prelude::*;

#[derive(Default)]
pub struct ZoomDropdown {
    // entry: RefCell<Option<gtk::Widget>>,
    button: RefCell<Option<gtk::Widget>>,
    entry: RefCell<Option<gtk::Widget>>,
    popover: RefCell<Option<gtk::Popover>>,
}

#[glib::object_subclass]
impl ObjectSubclass for ZoomDropdown {
    const NAME: &'static str = "ZoomDropdown";
    type Type = super::ZoomDropdown;
    type ParentType = gtk::Widget;

    fn class_init(klass: &mut Self::Class) {
        // The layout manager determines how child widgets are laid out.
        klass.set_layout_manager_type::<gtk::BoxLayout>();
    }

    fn new() -> Self {
        Self {
            button: RefCell::new(None),
            entry: RefCell::new(None),
            popover: RefCell::new(None),
        }
    }
}

impl ObjectImpl for ZoomDropdown {
    fn constructed(&self) {
        self.parent_constructed();

        let obj = self.obj();

        let layout_manager = obj
            .layout_manager()
            .and_downcast::<gtk::BoxLayout>()
            .unwrap();
        layout_manager.set_orientation(gtk::Orientation::Horizontal);

        let entry = gtk::Entry::builder()
            .vexpand(false)
            .text("100%")
            .build();

        let button = gtk::Button::builder()
            .icon_name("go-down-symbolic")
            .build();

        let popover  = {
            let label = gtk::Label::builder()
                .label("Popover")
                .valign(gtk::Align::Center)
                .halign(gtk::Align::Center)
                .build();

            gtk::Popover::builder()
                .width_request(180)
                .height_request(180)
                .halign(gtk::Align::Center)
                .autohide(false)
                .child(&label)
                .build()
        };       

        button.connect_clicked(glib::clone!(
            #[weak]
            popover,
            move |_| {
                if popover.is_visible() {
                    popover.popdown();
                } else {
                    popover.present();
                    popover.popup();
                }
            }
        ));

        entry.set_parent(&*obj);
        button.set_parent(&*obj);
        popover.set_parent(&*obj);

        self.entry.replace(Some(entry.upcast::<gtk::Widget>()));
        self.button.replace(Some(button.upcast::<gtk::Widget>()));
        self.popover.replace(Some(popover));

        obj.add_css_class("linked");

        
    }

    fn dispose(&self) {
        if let Some(child) = self.button.borrow_mut().take() {
            child.unparent();
        }

        if let Some(child) = self.entry.borrow_mut().take() {
            child.unparent();
        }

        if let Some(child) = self.popover.borrow_mut().take() {
            child.unparent();
        }
    }
}

impl WidgetImpl for ZoomDropdown {
    fn size_allocate(&self, width: i32, height: i32, baseline: i32) {
        self.parent_size_allocate(width, height, baseline);

        if let Some(popover) = self.popover.borrow().as_ref() {
            println!("width: {}", width);
            popover.set_size_request(width, -1);
            popover.queue_resize();
            popover.present();
        }
    }

    fn snapshot(&self, snapshot: &gtk::Snapshot) {
        self.parent_snapshot(snapshot);

        let obj = self.obj();
        let width = obj.width();
        let rec = obj.compute_bounds(obj.as_ref());

        println!("rec: {:?}", rec);

        if let Some(popover) = self.popover.borrow().as_ref() {
            popover.set_width_request(width);
            // popover.queue_resize();
            // popover.present();
        }
    }
}